######################################################################
#
# EPrints::Exporter
#
######################################################################
#
#  This file is part of GNU EPrints 2.
#  
#  Copyright (c) 2000-2004 University of Southampton, UK. SO17 1BJ.
#  
#  EPrints 2 is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  EPrints 2 is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with EPrints 2; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
######################################################################

=pod

=head1 NAME

B<EPrints::Exporter> - 

=head1 DESCRIPTION


=over 4

=cut

package EPrints::Exporter;


use strict;

######################################################################
=pod

=item $exporter = EPrints::Exporter->new( %properties )

Create a new exporter. 

=cut
######################################################################

sub new
{
	my( $class, %opts ) = @_;

	my $self = { opts=>{%opts} };
	bless $self, $class;

	if( $self->{opts}->{output} eq 'fh' )
	{
		if( !defined $self->{opts}->{fh} )
		{
			EPrints::Config::abort( 'fh exporter does not have fh set' );
		} 
	}
	elsif( $self->{opts}->{output} eq 'web' )
	{
		if( !defined $self->{opts}->{mimetype} )
		{
			EPrints::Config::abort( 'web exporter does not have mimetype set' );
		} 
		&SESSION->send_http_header( content_type=>$self->{opts}->{mimetype} );
	}
	else
	{
		EPrints::Config::abort( 'unsupported exporter output type: '.$self->{opts}->{output} );
	}

	return( $self );
}

######################################################################
=pod

=item $exporter->data( $data )

Do with the $data whetever this type of exporter does.

=cut
######################################################################

sub data
{
	my( $self, $data )= @_;

	if( $self->{opts}->{output} eq 'fh' )
	{
		print {$self->{opts}->{fh}} $data;
	}
	elsif( $self->{opts}->{output} eq 'web' )
	{
		print $data;	
	}
}

######################################################################
=pod

=item $exporter->finish()

Clean up and pass a return value if appropriate.

=cut
######################################################################

sub finish
{
	my( $self ) = @_;
}


1;
